﻿local F = CreateFrame("Frame", "BardToolsEventListener", UIParent)
BardTools = {}
local B = BardTools

function B:FullTargetName()
    local name, realm = UnitName("target")
    if (not realm) then
        realm = GetRealmName()
    end
    
    if (name and realm) then
        return name .. "-" .. realm
    end
end

do
    -- Code for timed functions
    local timedFunctions = {}
    local updateFunctionActive = false
    
    local f = CreateFrame("Frame", "BardToolsTimedFunctionHandler", UIParent)
    
    local function Update()
        local t = GetTime()
        for k,v in pairs(timedFunctions) do
            if (t > v[1]) then
                if (v[2]) then
                    v[2]()
                end
                tremove(timedFunctions, k)
                if (#timedFunctions == 0) then
                    f:SetScript("OnUpdate", nil)
                    updateFunctionActive = false
                end
            end
        end
    end
    
    function B:StartTimer(delay, func, name)
        tinsert(timedFunctions, {GetTime()+delay, func, name})
        
        if (not updateFunctionActive) then
            f:SetScript("OnUpdate", Update)
            updateFunctionActive = true
        end
    end
    
    function B:StopTimer(name)
        for k,v in pairs(timedFunctions) do
            if (v[3] == name) then
                v[1] = 0
                v[2] = nil
            end
        end
    end
    
    function B:StopAllTimers()
        wipe(timedFunctions)
        f:SetScript("OnUpdate", nil)
        updateFunctionActive = false
    end
end

function B:UnitRole(unit)
    -- Only intended to work with raid units
    local unitIndex = B:UnitIndex(unit)
    
    local combatRole = select(12, GetRaidRosterInfo(unitIndex))
    return combatRole
end

function B:UnitIndex(unit)
    unit = gsub(unit, "raid", "")
    
    return tonumber(unit)
end

function B:InitializeSavedVariables()
    if (not BardToolsData) then
        BardToolsData = {}
    end
    local b = BardToolsData
    
    -- Shadow-Lord Iskar stuff
    b.AnzuFrames = b.AnzuFrames or {}
    b.AnzuFrames.macro = b.AnzuFrames.macro or 2
    b.AnzuFrames.width = b.AnzuFrames.width or 38 -- 38
    b.AnzuFrames.height = b.AnzuFrames.height or 30 -- 30
    b.AnzuFrames.spacing = b.AnzuFrames.spacing or 1
    b.AnzuFrames.horizontalGrowthDirection = b.AnzuFrames.horizontalGrowthDirection or "Right"
    b.AnzuFrames.verticalGrowthDirection = b.AnzuFrames.verticalGrowthDirection or "Down"
    b.AnzuFrames.groupType = b.AnzuFrames.groupType or "Grid"
    b.AnzuFrames.anchorPoint = b.AnzuFrames.anchorPoint or {"TOP", UIParent, "TOP", 200, -300}
    b.AnzuFrames.showName = b.AnzuFrames.showName or false
end

function B:SetDispelDuty(unitName)
    BardToolsData.dispelDuty = unitName
    if (BardToolsData.dispelDuty) then
        print("BardTools: " .. BardToolsData.dispelDuty .. " on dispel duty.")
    else
        print("BardTools: Nobody on dispel duty.")
    end
    
    if (BardToolsConfig) then
        BardToolsDispelButtonLabelText:SetText(BardToolsData.dispelDuty or "No Dispel Duty")
    end
end

function B:BroadcastDispelDuty()
    if (UnitExists("target")) then
        local unitName = B:FullTargetName()
        B:SendAddonMessage("dispelduty:" .. unitName)
    else
        B:SendAddonMessage("removedispelduty")
    end
end

function B:SetInterruptDuty(unitName)
    BardToolsData.interruptDuty = unitName
    if (BardToolsData.interruptDuty) then
        print("BardTools: " .. BardToolsData.interruptDuty .. " on interrupt duty.")
    else
        print("BardTools: Nobody on interrupt duty.")
    end
    
    if (BardToolsConfig) then
        BardToolsInterruptButtonLabelText:SetText(BardToolsData.interruptDuty or "No Interrupt Duty")
    end
end

function B:BroadcastInterruptDuty()
    if (UnitExists("target")) then
        local unitName = B:FullTargetName()
        B:SendAddonMessage("interruptduty:" .. unitName)
    else
        B:SendAddonMessage("removeinterruptduty")
    end
end

function B:SendAddonMessage(msg)
    local channel
    if (IsInRaid()) then
        channel = "RAID"
    elseif (IsInGroup()) then
        channel = "PARTY"
    else
        local playerName = UnitName("player")
        SendAddonMessage("BardTools", msg, "WHISPER", playerName)
        return
    end
    
    SendAddonMessage("BardTools", msg, channel)
end

function B:ReceiveAddonMessage(msg, sender)
    if (strsub(msg, 1, 11) == "dispelduty:") then
        msg = strsub(msg, 12)
        B:SetDispelDuty(msg)
        
    elseif (strsub(msg, 1, 16) == "removedispelduty") then
        B:SetDispelDuty()
        
    elseif (strsub(msg, 1, 14) == "interruptduty:") then
        msg = strsub(msg, 15)
        B:SetInterruptDuty(msg)
        
    elseif (strsub(msg, 1, 19) == "removeinterruptduty") then
        B:SetInterruptDuty()
    end
end

function B:UpdateMouseoverUnit()
    if (UnitName("mouseover") == "Shadow-Lord Iskar" and UnitHealth("mouseover") / UnitHealthMax("mouseover") > 0.5 and not B.anzuHelperLoaded) then
        B:LoadAnzuModule()
    end
end

local function OnLoad()
    F:UnregisterEvent("ADDON_LOADED")
    F:RegisterEvent("CHAT_MSG_ADDON")
    F:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
    
    B:InitializeSavedVariables()
    
    RegisterAddonMessagePrefix("BardTools")
end

local function OnEvent(self, event, ...)
    if (event == "ADDON_LOADED" and ... == "BardTools") then
        OnLoad()
    elseif (event == "CHAT_MSG_ADDON" and ... == "BardTools") then
        local _, msg, _, sender = ...
        B:ReceiveAddonMessage(msg, sender)
    elseif (event == "UPDATE_MOUSEOVER_UNIT") then
        B:UpdateMouseoverUnit()
    end
end

function B:Slash(cmd)
    if (cmd == "dispel") then
        B:BroadcastDispelDuty()
    elseif (cmd == "interrupt") then
        B:BroadcastInterruptDuty()
    else
        B:OpenConfig()
    end
end

SLASH_BARDTOOLS1 = "/bardtools";
SlashCmdList["BARDTOOLS"] = function(cmd) B:Slash(cmd) end

F:SetScript("OnEvent", OnEvent)
F:RegisterEvent("ADDON_LOADED")